/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2015, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** KC           Kay Claussen                                                  **
** BG           Blandine Guillot                                              **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2016-10-13, KC:   Initial version                                  **
** V1.0.1: 2020-02-27, BG:   Updated abstract                                 **
*******************************************************************************/
 
/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** LIN: Detection of automatic baudrate                                       **
********************************************************************************
** Timer 2 is configured to capture the LIN baudrate. Read out of baudrate    **
** is done after synch field detection in LIN_End_Of_Synch_Handler().         **
** Device generates interrupt on reception of                                 **
** - LIN Synch Field -> P0.1 toggled                                          **
** - LIN Synch Field errors -> P0.2 toggled                                   **
** - T2 Interrupt on falling edge of LIN signal-> P0.3 toggled                **
** - UART byte reception -> P1.0 toggled                                      **
** Variable LIN_baudrate_ticks keeps the captured time of 8 bits.             **
** With TLE9869QXA20: LIN_baudrate_ticks ~ 10015 ticks at baudrate 19200      **
**                    f_sys = 24 MHz => T_sys = 1/24000000 ~ 42 ns/tick       **
**                    => 10015[tick] x 42[ns/tick] ~ 417292 ns for 8 bits     **
**                    => 417292[ns/8bits] / 8[bit] ~ 52.16 us for 1 bit       **
**                    => 1[bit] / 0.00005216[s] ~ 19171 bit/s = 19171 baud    **
*******************************************************************************/

#include "tle_device.h"

#define ERRSYNC_EOFSYNC_BRK 0x38 /* mask for lin break, synch and synch error */

static uint16 LIN_baudrate_ticks;

int main(void)
{
  /*****************************************************************************
  ** Initialization of the hardware modules based on the configuration set    **
  ** in Config Wizard                                                         **
  *****************************************************************************/
  TLE_Init();

  for (;;)
  {
    (void)WDT1_Service();
  }
}


/* ISR for LIN End of Synch Field detection */
void LIN_End_Of_Synch_Handler(void)
{
    /* Toggle P0.1 */
    PORT_P01_Output_Toggle_Set(); 
    /* If break field AND synch field AND no sync error appeared, then ... */
    if ((SCU->LINST.reg & ERRSYNC_EOFSYNC_BRK) == 0x18) 
    {
      /* ... capture timer value */
      LIN_baudrate_ticks = TIMER2_Get_Capture(); 
    }
}


/* ISR for LIN synch filed error detection */
void LIN_Synch_Error_Handler(void)
{
    /* Toggle P0.2 */
    PORT_P02_Output_Toggle_Set(); 
}


/* ISR for Timer2 */
void TIMER2_Handler(void)
{
    /* Toggle P0.3 */
    PORT_P03_Output_Toggle_Set(); 
    TIMER2_Stop();
    TIMER2_Clear_Count();
    /* Clear  LIN field indicators */
    SCU->LINSCLR.reg = ERRSYNC_EOFSYNC_BRK;  
    /* Disable external Timer2 interrupt */
    TIMER2_External_Int_Dis();    
}


/* ISR for UART Receive */
void UART_Receive_Handler(void)
{
    /* Toggle P1.0 */
    PORT_P10_Output_Toggle_Set(); 
    /* In a LIN driver this part of the code should be located after reception */
    /* of the last LIN byte or after the occurance of an error on LIN.         */
    /* This will reduce the number of T2 interrupts.                           */
    
    /* Clear pending T2 interrupt */
    TIMER2->T2ICLR.reg = 0x40;  
    /* Enable external T2 interrupt */
    TIMER2_External_Int_En();     
}
