/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2015, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** BG           Blandine Guillot                                              **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2020-02-27, BG:   Initial version of revision history              **
*******************************************************************************/
 
/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** Flash: Program one page in data flash                                      **
********************************************************************************
** Reading the first data flash page after page erase leads to an NMI.        **
** Inside the NMI ISR, the first data flash page will be filled with data     **
** consecutive read inside the main application.                              **
** Now it is possible to read from them from the data flash.                  **
*******************************************************************************/

#include "tle_device.h"
#include "eval_board.h"

/* Allocate one flash page in RAM */
static uint8 TargetData[FlashPageSize];
static int NVM_Error;

int main(void)
{
  int i;
  uint8 *p;
  static uint16 checksum, data;

  /*****************************************************************************
  ** Initialization of the hardware modules based on the configuration set    **
  ** in Config Wizard                                                         **
  *****************************************************************************/
  TLE_Init();

  checksum = 0;
  /* Fill up the TargetData array with housenumbers */
  for (i = 0; i < FlashPageSize; i++)
  {
    TargetData[i] = (uint8) i;
    checksum += (uint8) i;
  }

  /* Erase Data Flash sector */
  NVM_Error = USER_ERASE_SECTOR(DataFlashStart);

  /* Check if Sector Erase passed */
  if (NVM_Error == 0u)
  {
    /* Perform a read access to the first Data Flash page */
    /* Given the logical page 0 is not mapped to any physical flash page, a NMI will be issued */
    p = (uint8 *) DataFlashStart;
    /* Read DataFlashStart + Offset (0) */
    data = (uint16) p[0];
    /* NMI ISR gets called, page 0 gets loaded with data */

    /* Check if preloading of data inside the NMI ISR did work */
    if (NVM_Error == 0u)
    {
      /* Read DataFlashStart + Offset (0): you should be able to read data this time */
      data = (uint16) 0;
      for (i = 0; i < FlashPageSize; i++)
      {
        data += (uint16) p[i];
      }
      if (data == checksum)
      {
        /* Data in Data Flash page 0 correct */
        PORT_ChangePin(LED6, PORT_ACTION_SET);
      }
      else
      {
        /* Data in Data Flash page 0 wrong */
        PORT_ChangePin(LED7, PORT_ACTION_SET);
      }
    }
    else
    {
      /* An error happened during Page Write in the NMI ISR */
      PORT_ChangePin(LED7, PORT_ACTION_SET);
    }
  }
  else
  {
    /* An error happened during Erase Sector */
    PORT_ChangePin(LED7, PORT_ACTION_SET);
  }

  for (;;)
  {
    (void)WDT1_Service();

  }
}

void NVM_Handle_PageFault(void)
{
  /* Fill up first page with some data */
  /* The TargetData array will be written to page 0 */

  NVM_Error = ProgramPage(DataFlashStart, TargetData, 0, 0, 0);
}
