/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2015, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** BG           Blandine Guillot                                              **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2020-02-27, BG:   Initial version of revision history              **
*******************************************************************************/
 
/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** Flash: Program a page when the write protection is set                     **
********************************************************************************
** Read the customer id to determine the design step.                         **
** Depending on the design step, the correct method is applied to set the     **
** protection.                                                                **
** A page write is performed in the protected NVM region. This operation      **
** should failed due to the protection. This is displayed with LED6 turned on.**
*******************************************************************************/

#include "tle_device.h"
#include "eval_board.h"

/* Allocate one flash page in RAM */
static uint8 TargetData[FlashPageSize];
static int NVM_Error;
static TCustomerID CustomerID;

int main(void)
{
  int i;
  /*****************************************************************************
  ** Initialization of the hardware modules based on the configuration set    **
  ** in Config Wizard                                                         **
  *****************************************************************************/
  TLE_Init();

  /* Set the protection according to the design step */
  GetCustomerID((TCustomerID *) & CustomerID);

  switch (CustomerID.bit.Step)
  {
  case 0xAA:
  case 0xAB:
  case 0xBA:
    {
      SCU_ChangeNVMProtection(NVM_DATA_WRITE, PROTECTION_SET);
    }
    break;
  case 0xBB:
  case 0xBC:
  case 0xBE:
  case 0xBF:
    {
      /* Enable write protection */
      USER_DFLASH_WR_PROT_EN(0x0000);
      /* Disable write protection */
      //USER_DFLASH_WR_PROT_DIS(0x0000);
    }
    break;
  default:
    break;
  }

  /* Fill up the TargetData array with housenumbers */
  for (i = 0; i < FlashPageSize; i++)
  {
    TargetData[i] = (uint8) i;
  }

  /* Call ProgramPage routine */
  /* If the write protection is enabled, the ProgramPage routine returns an error */
  NVM_Error = ProgramPage(DataFlashStart, TargetData, 0, 0, 0);

  /* Check if the programming worked fine */
  if (NVM_Error == 0u)
  {
    /* Data in page 0 of the Data Flash is correct */
    /* This is not expected in this example */
    PORT_ChangePin(LED7, PORT_ACTION_SET);
  }
  else
  {
    /* An error happened during ProgramPage */
    /* This is expected in this example as the data flash is protected */
    PORT_ChangePin(LED6, PORT_ACTION_SET);
  }

  for (;;)
  {
    (void)WDT1_Service();
  }
}
