/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2015, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/
/**
 * \file     sdadc.h
 *
 * \brief    Sigma Delta ADC low level access library
 *
 * \version  V0.1.4
 * \date     28. Feb 2020
 */

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** DM           Daniel Mysliwitz                                              **
** JO           Julia Ott                                                     **
** BG           Blandine Guillot                                              **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V0.1.0: 2017-11-07, DM:   Initial version                                  **
**                           MISRA 2012 compliance, the following PC-Lint     **
**                           rules are globally deactivated:                  **
**                           - Info 793: ANSI/ISO limit of 6 'significant     **
**                             characters in an external identifier           **
**                           - Info 835: A zero has been given as right       **
**                             argument to operator                           **
**                           - Info 845: The left argument to operator '&'    **
**                             is certain to be 0                             **
** V0.1.1: 2017-11-21, DM:   ADC34 prefix in API function replaced by SDADC   **
** V0.1.2: 2018-11-27, JO:   Doxygen update, moved revision history from      **
**                           sdadc.c to sdadc.h                               **
** V0.1.3: 2020-02-20, BG:   Updated return value type in                     **
**                           SDADC_Ch{0/1}_Result_Get()                       **
** V0.1.4: 2020-02-28, BG:   Updated revision history format                  **
*******************************************************************************/

#ifndef SDADC_H
#define SDADC_H

/*******************************************************************************
**                                  Includes                                  **
*******************************************************************************/
#include "tle986x.h"
#include "types.h"
#include "sfr_access.h"

/*******************************************************************************
**                      Global Constant Declarations                          **
*******************************************************************************/

/*******************************************************************************
**                        Global Type Declarations                            **
*******************************************************************************/
/** \enum TSDADC_OSR
 *  \brief This enum lists the SDADC Oversampling Ratio.
 */
typedef enum
{
  SDADC_OSR_128    = 7,  /**< SDADC Oversampling Ratio - 128 */
  SDADC_OSR_256    = 8,  /**< SDADC Oversampling Ratio - 256 */
  SDADC_OSR_512    = 9,  /**< SDADC Oversampling Ratio - 512 */
  SDADC_OSR_1024   = 10, /**< SDADC Oversampling Ratio - 1024 */
  SDADC_OSR_2048   = 11  /**< SDADC Oversampling Ratio - 2048 */
} TSDADC_OSR;

/** \enum TSDADC_DITHVAL
 *  \brief This enum lists the ADC34 Dither Value.
 */
typedef enum
{
  SDADC_DITH_0_mV     = 0,  /**< ADC34 Dither Value - 0 mV */
  SDADC_DITH_12_5_mV  = 1,  /**< ADC34 Dither Value - 12.5 mV */
  SDADC_DITH_25_0_mV  = 2,  /**< ADC34 Dither Value - 25 mV */
  SDADC_DITH_37_5_mV  = 3,  /**< ADC34 Dither Value - 37.5 mV */
  SDADC_DITH_50_0_mV  = 4,  /**< ADC34 Dither Value - 50 mV */
  SDADC_DITH_62_5_mV  = 5,  /**< ADC34 Dither Value - 62.5 mV */
  SDADC_DITH_75_0_mV  = 6,  /**< ADC34 Dither Value - 75 mV */
  SDADC_DITH_87_5_mV  = 7,  /**< ADC34 Dither Value - 87.5 mV */
  SDADC_DITH_100_0_mV = 8,  /**< ADC34 Dither Value - 100 mV */
  SDADC_DITH_112_5_mV = 9,  /**< ADC34 Dither Value - 112.5 mV */
  SDADC_DITH_125_0_mV = 10, /**< ADC34 Dither Value - 125 mV */
  SDADC_DITH_137_5_mV = 11, /**< ADC34 Dither Value - 137.5 mV */
  SDADC_DITH_150_0_mV = 12, /**< ADC34 Dither Value - 150 mV */
  SDADC_DITH_162_5_mV = 13, /**< ADC34 Dither Value - 162.5 mV */
  SDADC_DITH_175_0_mV = 14, /**< ADC34 Dither Value - 175 mV */
  SDADC_DITH_187_5_mV = 15  /**< ADC34 Dither Value - 187.5 mV */
} TSDADC_DITHVAL;

/** \enum TSDADC_EOC_CNT
 *  \brief This enum lists the ADC34 EoC counter.
 */
typedef enum
{
  SDADC_IRQ_AFTER_EVERY_EOC = 0, /**< ADC34 EoC counter - Every */
  SDADC_IRQ_AFTER_8TH_EOC   = 1, /**< ADC34 EoC counter - 8TH */
  SDADC_IRQ_AFTER_16TH_EOC  = 2, /**< ADC34 EoC counter - 16TH */
  SDADC_IRQ_AFTER_32ND_EOC  = 3  /**< ADC34 EoC counter - 32ND */
} TSDADC_EOC_CNT;

/*******************************************************************************
**                     Global Inline Function Definitions                     **
*******************************************************************************/
/** \brief enables ADC3 analog frontend.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC3 analog frontend.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SDADC_Ch0_Power_On();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_Ch0_Power_On(void)
{
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC3_EN_Pos, (uint32)ADC34_CTRL_STS_ADC3_EN_Msk, 1u);
}

/** \brief disables ADC3 analog frontend and ADC3 conversion.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables ADC3 analog frontend.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SDADC_Ch0_Power_Off();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_Ch0_Power_Off(void)
{
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC3_EN_Pos, (uint32)ADC34_CTRL_STS_ADC3_EN_Msk, 0u);
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC3_SOC_Pos, (uint32)ADC34_CTRL_STS_ADC3_SOC_Msk, 0u);
}

/** \brief enables ADC4 analog frontend.
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC4 analog frontend.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SDADC_Ch1_Power_On();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_Ch1_Power_On(void)
{
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC4_EN_Pos, (uint32)ADC34_CTRL_STS_ADC4_EN_Msk, 1u);
}

/** \brief disables ADC4 analog frontend and ADC4 conversion.
 *
 * \brief <b>Example</b><br>
 * \brief This example disables ADC4 analog frontend.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SDADC_Ch1_Power_Off();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_Ch1_Power_Off(void)
{
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC4_EN_Pos, (uint32)ADC34_CTRL_STS_ADC4_EN_Msk, 0u);
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC4_SOC_Pos, (uint32)ADC34_CTRL_STS_ADC4_SOC_Msk, 0u);
}

/** \brief gets ADC3 Result value
 *
 * \return ADC3 Result
 *
 * \brief <b>Example</b><br>
 * \brief This example reads ADC3 Result value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     sint16 sts;
 *
 *     SDADC_Ch0_Power_On();
 *     sts = SDADC_Ch0_Result_Get();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE sint16 SDADC_Ch0_Result_Get(void)
{
  return ((sint16)ADC34->RESU.bit.ADC3_RESU);
}

/** \brief gets ADC4 Result value
 *
 * \return ADC4 Result
 *
 * \brief <b>Example</b><br>
 * \brief This example reads ADC4 Result value.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     sint16 sts;
 *
 *     SDADC_Ch1_Power_On();
 *     sts = SDADC_Ch1_Result_Get();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE sint16 SDADC_Ch1_Result_Get(void)
{
  return ((sint16)ADC34->RESU.bit.ADC4_RESU);
}

/** \brief selects ADC3 Oversampling Ratio
 *
 * \param osr ADC3 Oversampling Ratio
 *
 * \brief <b>Example</b><br>
 * \brief This example configures ADC3 Oversampling Ratio to 256.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     SDADC_Ch0_Power_On();
 *     SDADC_Ch0_OverSamplingRatio_Sel(8u);
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_Ch0_OverSamplingRatio_Sel(uint32 osr)
{
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC3_OSR_Pos, (uint32)ADC34_CTRL_STS_ADC3_OSR_Msk, osr);
}

/** \brief selects ADC4 Oversampling Ratio
 *
 * \param osr ADC4 Oversampling Ratio
 *
 * \brief <b>Example</b><br>
 * \brief This example configures ADC4 Oversampling Ratio to 1024.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     SDADC_Ch1_Power_On();
 *     SDADC_Ch1_OverSamplingRatio_Sel(10u);
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_Ch1_OverSamplingRatio_Sel(uint32 osr)
{
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC4_OSR_Pos, (uint32)ADC34_CTRL_STS_ADC4_OSR_Msk, osr);
}

/** \brief enables ADC34 Dithering
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC34 Dithering.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     SDADC_Dithering_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_Dithering_En(void)
{
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC34_DITHEN_Pos, (uint32)ADC34_CTRL_STS_ADC34_DITHEN_Msk, 1u);
}

/** \brief disables ADC34 Dithering
 *
 * \brief <b>Example</b><br>
 * \brief This example disables ADC34 Dithering.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     SDADC_Dithering_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_Dithering_Dis(void)
{
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC34_DITHEN_Pos, (uint32)ADC34_CTRL_STS_ADC34_DITHEN_Msk, 0u);
}

/** \brief selects ADC34 Dithering value
 *
 * \param dith ADC34 Dithering value
 *
 * \brief <b>Example</b><br>
 * \brief This example sets ADC34 Dithering value to 1.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     SDADC_Dithering_Sel(1u);
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_Dithering_Sel(uint32 dith)
{
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC34_DITHVAL_Pos, (uint32)ADC34_CTRL_STS_ADC34_DITHVAL_Msk, dith);
}

/** \brief selects ADC34 EoC counter (IRQ rate of ADC34)
 *
 * \param rate EoC counter
 *
 * \brief <b>Example</b><br>
 * \brief This example sets ADC34 EoC counter to IRQ with every EoC.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     SDADC_IRQ_Rate_Sel(0u);
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_IRQ_Rate_Sel(uint32 rate)
{
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC34_EoC_CNT_Pos, (uint32)ADC34_CTRL_STS_ADC34_EoC_CNT_Msk, rate);
}

/** \brief enables ADC3 conversion
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC3 conversion.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     SDADC_Ch0_Power_On();
 *     SDADC_Ch0_SOC_Set();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_Ch0_SOC_Set(void)
{
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC3_SOC_Pos, (uint32)ADC34_CTRL_STS_ADC3_SOC_Msk, 1u);
}

/** \brief disables ADC3 conversion
 *
 * \brief <b>Example</b><br>
 * \brief This example disables ADC3 conversion.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     SDADC_Ch0_Power_On();
 *     SDADC_Ch0_SOC_Rst();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_Ch0_SOC_Rst(void)
{
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC3_SOC_Pos, (uint32)ADC34_CTRL_STS_ADC3_SOC_Msk, 0u);
}

/** \brief enables ADC4 conversion
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC4 conversion.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     SDADC_Ch1_Power_On();
 *     SDADC_Ch1_SOC_Set();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_Ch1_SOC_Set(void)
{
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC4_SOC_Pos, (uint32)ADC34_CTRL_STS_ADC4_SOC_Msk, 1u);
}

/** \brief disables ADC4 conversion
 *
 * \brief <b>Example</b><br>
 * \brief This example disables ADC4 conversion.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     SDADC_Ch1_Power_On();
 *     SDADC_Ch1_SOC_Rst();
 *
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_Ch1_SOC_Rst(void)
{
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC4_SOC_Pos, (uint32)ADC34_CTRL_STS_ADC4_SOC_Msk, 0u);
}

/** \brief enables ADC3 and ADC4 conversions
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC3 and ADC4 conversions.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     SDADC_Ch0_Power_On();
 *     SDADC_Ch1_Power_On();
 *     SDADC_SOC_Set();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_SOC_Set(void)
{
  ADC34->CTRL_STS.reg |= ((uint32)1 << (uint32)ADC34_CTRL_STS_ADC4_SOC_Pos) | ((uint8)1 << (uint8)ADC34_CTRL_STS_ADC3_SOC_Pos);
}

/** \brief disables ADC3 and ADC4 conversions
 *
 * \brief <b>Example</b><br>
 * \brief This example disables ADC3 and ADC4 conversions.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     SDADC_Ch0_Power_On();
 *     SDADC_Ch1_Power_On();
 *     SDADC_SOC_Rst();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_SOC_Rst(void)
{
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC3_SOC_Pos, (uint32)ADC34_CTRL_STS_ADC3_SOC_Msk, 0u);
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC4_SOC_Pos, (uint32)ADC34_CTRL_STS_ADC4_SOC_Msk, 0u);
}

/** \brief reads ADC3 End of Conversion Status
 *
 * \retval 1 ADC3 conversion terminated
 * \retval 0 ADC3 conversion still in progress
 *
 * \brief <b>Example</b><br>
 * \brief This example waits until the End of ADC3 Conversion.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     SDADC_Ch0_Power_On();
 *     while(SDADC_Ch0_EOC_Sts() == 0)
 *     {
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE uint8 SDADC_Ch0_EOC_Sts(void)
{
  return ( u1_Field_Rd32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC3_EoC_STS_Pos, (uint32)ADC34_CTRL_STS_ADC3_EoC_STS_Msk) );
}

/** \brief reads ADC4 End of Conversion Status
 *
 * \retval 1 ADC4 conversion terminated
 * \retval 0 ADC4 conversion still in progress
 *
 * \brief <b>Example</b><br>
 * \brief This example waits until the End of ADC4 Conversion.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     SDADC_Ch1_Power_On();
 *     while(SDADC_Ch1_EOC_Sts() == 0)
 *     {
 *     }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE uint8 SDADC_Ch1_EOC_Sts(void)
{
  return ( u1_Field_Rd32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC4_EoC_STS_Pos, (uint32)ADC34_CTRL_STS_ADC4_EoC_STS_Msk) );
}

/** \brief sets ADC34 reference voltage to VAREF
 *
 * \brief <b>Example</b><br>
 * \brief This example sets ADC34 reference voltage to VAREF.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     SDADC_VAREF_Set();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_VAREF_Set(void)
{
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC34_REF_SEL_Pos, (uint32)ADC34_CTRL_STS_ADC34_REF_SEL_Msk, 0u);
}

/** \brief sets ADC34 reference voltage to VDDEXT
 *
 * \brief <b>Example</b><br>
 * \brief This example sets ADC34 reference voltage to VDDEXT.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     SDADC_VDDEXT_Set();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_VDDEXT_Set(void)
{
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC34_REF_SEL_Pos, (uint32)ADC34_CTRL_STS_ADC34_REF_SEL_Msk, 1u);
}

/** \brief enables ADC3 Offset Measurement
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC3 Offset Measurement.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     SDADC_Ch0_Power_On();
 *     SDADC_Ch0_Offset_Meas_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_Ch0_Offset_Meas_En(void)
{
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC3_OFS_MEAS_EN_Pos, (uint32)ADC34_CTRL_STS_ADC3_OFS_MEAS_EN_Msk, 1u);
}

/** \brief disables ADC3 Offset Measurement
 *
 * \brief <b>Example</b><br>
 * \brief This example disables ADC3 Offset Measurement.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     SDADC_Ch0_Power_On();
 *     SDADC_Ch0_Offset_Meas_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_Ch0_Offset_Meas_Dis(void)
{
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC3_OFS_MEAS_EN_Pos, (uint32)ADC34_CTRL_STS_ADC3_OFS_MEAS_EN_Msk, 0u);
}

/** \brief enables ADC4 Offset Measurement
 *
 * \brief <b>Example</b><br>
 * \brief This example enables ADC4 Offset Measurement.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     SDADC_Ch1_Power_On();
 *     SDADC_Ch1_Offset_Meas_En();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_Ch1_Offset_Meas_En(void)
{
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC4_OFS_MEAS_EN_Pos, (uint32)ADC34_CTRL_STS_ADC4_OFS_MEAS_EN_Msk, 1u);
}

/** \brief disables ADC4 Offset Measurement
 *
 * \brief <b>Example</b><br>
 * \brief This example disables ADC4 Offset Measurement.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *     SDADC_Ch1_Power_On();
 *     SDADC_Ch1_Offset_Meas_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_Ch1_Offset_Meas_Dis(void)
{
  Field_Mod32(&ADC34->CTRL_STS.reg, (uint32)ADC34_CTRL_STS_ADC4_OFS_MEAS_EN_Pos, (uint32)ADC34_CTRL_STS_ADC4_OFS_MEAS_EN_Msk, 0u);
}

/** \brief clears ADC3 EOC Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC3 EOC Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SDADC_Ch0_Int_En();
 *   if (SCUPM->SYS_IS.bit.ADC3_EOC_IS == 1u)
 *   {
 *     ADC34_ADC3_CALLBACK();
 *     SDADC_Ch0_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_Ch0_Int_Clr(void)
{
  Field_Wrt32(&SCUPM->SYS_ISCLR.reg, (uint32)SCUPM_SYS_ISCLR_ADC3_EOC_ICLR_Pos, (uint32)SCUPM_SYS_ISCLR_ADC3_EOC_ICLR_Msk, 1u);
}

/** \brief clears ADC4 EOC Interrupt flag.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC4 EOC Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SDADC_Ch1_Int_En();
 *   if (SCUPM->SYS_IS.bit.ADC4_EOC_IS == 1u)
 *   {
 *     ADC34_ADC4_CALLBACK();
 *     SDADC_Ch1_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_Ch1_Int_Clr(void)
{
  Field_Wrt32(&SCUPM->SYS_ISCLR.reg, (uint32)SCUPM_SYS_ISCLR_ADC4_EOC_ICLR_Pos, (uint32)SCUPM_SYS_ISCLR_ADC4_EOC_ICLR_Msk, 1u);
}

/** \brief enables ADC3 EOC Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC3 EOC Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SDADC_Ch0_Int_En();
 *   if (SCUPM->SYS_IS.bit.ADC3_EOC_IS == 1u)
 *   {
 *     ADC34_ADC3_CALLBACK();
 *     SDADC_Ch0_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_Ch0_Int_En(void)
{
  Field_Mod32(&SCUPM->SYS_IRQ_CTRL.reg, (uint32)SCUPM_SYS_IRQ_CTRL_ADC3_EOC_IE_Pos, (uint32)SCUPM_SYS_IRQ_CTRL_ADC3_EOC_IE_Msk, 1u);
}

/** \brief disables ADC3 EOC Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC3 EOC Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SDADC_Ch0_Int_En();
 *   if (SCUPM->SYS_IS.bit.ADC3_EOC_IS == 1u)
 *   {
 *     ADC34_ADC3_CALLBACK();
 *     SDADC_Ch0_Int_Clr();
 *   }
 *   SDADC_Ch0_Int_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_Ch0_Int_Dis(void)
{
  Field_Mod32(&SCUPM->SYS_IRQ_CTRL.reg, (uint32)SCUPM_SYS_IRQ_CTRL_ADC3_EOC_IE_Pos, (uint32)SCUPM_SYS_IRQ_CTRL_ADC3_EOC_IE_Msk, 0u);
}

/** \brief enables ADC4 EOC Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC4 EOC Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SDADC_Ch1_Int_En();
 *   if (SCUPM->SYS_IS.bit.ADC4_EOC_IS == 1u)
 *   {
 *     ADC34_ADC4_CALLBACK();
 *     SDADC_Ch1_Int_Clr();
 *   }
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_Ch1_Int_En(void)
{
  Field_Mod32(&SCUPM->SYS_IRQ_CTRL.reg, (uint32)SCUPM_SYS_IRQ_CTRL_ADC4_EOC_IE_Pos, (uint32)SCUPM_SYS_IRQ_CTRL_ADC4_EOC_IE_Msk, 1u);
}

/** \brief disables ADC4 EOC Interrupt.
 *
 * \brief <b>Example</b><br>
 * \brief This example treats the ADC4 EOC Interrupt.
 * ~~~~~~~~~~~~~~~{.c}
 * void Example_Function(void)
 * {
 *   SDADC_Ch1_Int_En();
 *   if (SCUPM->SYS_IS.bit.ADC4_EOC_IS == 1u)
 *   {
 *     ADC34_ADC4_CALLBACK();
 *     SDADC_Ch1_Int_Clr();
 *   }
 *   SDADC_Ch1_Int_Dis();
 * }
 * ~~~~~~~~~~~~~~~
 * \ingroup sdadc_api
 */
INLINE void SDADC_Ch1_Int_Dis(void)
{
  Field_Mod32(&SCUPM->SYS_IRQ_CTRL.reg, (uint32)SCUPM_SYS_IRQ_CTRL_ADC4_EOC_IE_Pos, (uint32)SCUPM_SYS_IRQ_CTRL_ADC4_EOC_IE_Msk, 0u);
}

/*******************************************************************************
**                        Global Function Declarations                        **
*******************************************************************************/
/** \brief Initializes the SDADC module (ADC3 and ADC4).
 *
 * \ingroup sdadc_api
 */
void SDADC_Init(void);

/*******************************************************************************
**                     Global Inline Function Definitions                     **
*******************************************************************************/

#endif
